@@ -17,16 +17,12 @@ void __sdata_info(const char *fmt, ...)
 	struct va_format vaf = {
 		.fmt = fmt,
 	};
-	va_list args, args2;
+	va_list args;
 
 	va_start(args, fmt);
-
-	va_copy(args2, args);
-	vaf.va = &args2;
-	pr_info("%pV", &vaf);
-	va_end(args2);
-
 	vaf.va = &args;
+
+	pr_info("%pV", &vaf);
 	trace_mac80211_info(&vaf);
 	va_end(args);
 }
@@ -39,16 +35,10 @@ void __sdata_dbg(bool print, const char *fmt, ...)
 	va_list args;
 
 	va_start(args, fmt);
-
-	if (print) {
-		va_list args2;
-
-		va_copy(args2, args);
-		vaf.va = &args2;
-		pr_debug("%pV", &vaf);
-		va_end(args2);
-	}
 	vaf.va = &args;
+
+	if (print)
+		pr_debug("%pV", &vaf);
 	trace_mac80211_dbg(&vaf);
 	va_end(args);
 }
@@ -58,16 +48,12 @@ void __sdata_err(const char *fmt, ...)
 	struct va_format vaf = {
 		.fmt = fmt,
 	};
-	va_list args, args2;
+	va_list args;
 
 	va_start(args, fmt);
-
-	va_copy(args2, args);
-	vaf.va = &args2;
-	pr_err("%pV", &vaf);
-	va_end(args2);
-
 	vaf.va = &args;
+
+	pr_err("%pV", &vaf);
 	trace_mac80211_err(&vaf);
 	va_end(args);
 }
@@ -80,16 +66,10 @@ void __wiphy_dbg(struct wiphy *wiphy, bool print, const char *fmt, ...)
 	va_list args;
 
 	va_start(args, fmt);
-
-	if (print) {
-		va_list args2;
-
-		va_copy(args2, args);
-		vaf.va = &args2;
-		pr_debug("%pV", &vaf);
-		va_end(args2);
-	}
 	vaf.va = &args;
+
+	if (print)
+		wiphy_dbg(wiphy, "%pV", &vaf);
 	trace_mac80211_dbg(&vaf);
 	va_end(args);
 }
